#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>

#include "util.h"

uint8_t rev[256];

int bitcmp(uint8_t *a, int off_a, uint8_t *b, int off_b, int n_bits) {
	int i,av,bv;
	for (i=0; i< n_bits; i++) {
		av = bit_is_set(a, off_a+i);
		bv = bit_is_set(b, off_b+i);
		if (av && !bv) return 1;
		if (!av && bv) return -1;
	}
	return 0;
}

void build_rev() {
	int i,v,j;
	for (i=0; i< 256; i++) {
		v = 0;
		for (j = 0; j < 8; j++) {
			if (i & (1<<j)) v |= (0x80 >> j);
		}
		rev[i] = v;
	}
}


void dump(uint8_t *buf, uint32_t addr, int len, int rev_bit, FILE *fp) {
	int i,n;
	int match = 0;

	if (!rev[1]) build_rev();

	fprintf(fp, "*** dump ***\n");
	addr &= ~0xf;
	len = (len + 15) >> 4;
	for (n=0; n<len; n++) {
		if ((n!=0) && !memcmp(buf+addr, buf+addr -16, 16)) {
			if (!match) {
				fprintf(fp, " * \n");
			}
			match = 1;
			addr += 16;
			continue;
		}
		match = 0;
		fprintf(fp, "%04x:",addr);
		for (i=0; i<16; i++) {
			fprintf(fp, " %02x", rev_bit ? rev[buf[addr+i]]
						: buf[addr+i]);
		}
		fprintf(fp, "\n");
		addr += 16;
	}
	fprintf(fp, "****************\n");
}

#if defined (WIN32NATIVE)
#include <windows.h>

static LARGE_INTEGER freq;
static int has_highperf;

static void dtime_init()
{
	SetThreadAffinityMask( GetCurrentThread(), 1 );
	Sleep(1);
        has_highperf=QueryPerformanceFrequency(&freq);
//printf("Freq = %10.3f\n", (double)freq.QuadPart);

}

double dtime()
{
	LARGE_INTEGER time_val;

    if (!has_highperf) dtime_init();
    if (has_highperf) {
	QueryPerformanceCounter(&time_val);
	return (double)time_val.QuadPart / (double)freq.QuadPart;
    }
    return 0.0;
}
#endif

static double ps;
static int ptotal,pcur;
void progress_start(char *msg, int total) {
	ps = dtime();
	ptotal = total;
	pcur = 0;
	printf("  %10s :",msg);
	fflush(stdout);
}

void progress(int cur) {
	int i,n;
	n = (double)(51.0 * cur)/ ptotal;
	if (n > 50) n = 50;
	if (n > pcur) {
		for (i=pcur; i< n; i++) {
			printf("#");
		}
		pcur = n;
		fflush(stdout);
	}
}

void progress_stop() {
	double pe;
	pe = dtime();
	printf("%7.3f sec\n", (pe - ps));
	fflush(stdout);
}

