`define TARGET_MachXO2
//`define TARGET_MachXO

`ifdef TARGET_MachXO
   `define JTAG_BSCAN JTAGD
   `define HAVE_OSCC
//	module OSCC (OSC); //synthesis syn_black_box
//	  output OSC;
//	endmodule

`elsif TARGET_MachXO2
   `define JTAG_BSCAN JTAGF

   `define HAVE_OSCH
//	module OSCH (
//	   input    STDBY,
//	   output   OSC, SEDSTDBY ); //synthesis syn_black_box
//
//	   parameter NOM_FREQ = "2.08";
//	endmodule
`endif

module tool_test (
		// JTAG 
		  input TOP_TDI
		, input TOP_TCK
		, input TOP_TMS
		, output  TOP_TDO

		// LED
		,  output [7:0] LED
`ifdef TARGET_MachXO2
		// Serial
		, input TXD, RTS, DTR
		, inout RXD, CTS
		// External OSC
		, input EXTOSC
		, inout EXTOSC_EN
`endif
		);

`ifdef TARGET_MachXO2
  wire RXD = 1'bz;
  wire CTS = 1'bz;
  wire EXTOSC_EN = 1'bz;
`endif

  `JTAG_BSCAN bscan (
`ifdef TARGET_MachXO
	.JRST(BSCAN_RESET) 
`elsif TARGET_MachXO2
	.JRSTN(BSCAN_RESET_N) 
`endif
	, .JSHIFT(BSCAN_SHIFT)
//	, .JUPDATE(BSCAN_UPDATE)

	, .TDI(TOP_TDI)
	, .TCK(TOP_TCK)
	, .TMS(TOP_TMS) 
	, .TDO(TOP_TDO)

	, .JTDI(JTDI_IN)
	, .JTCK(JTCK_IN) // negedge capt, posedge shift

	// signals for IPA
	, .JRTI1(JRTI1)
	, .JCE1(SEL1)
	, .JTDO1(JTDO1)

	// signals for IPB
	, .JRTI2(JRTI2)
	, .JCE2(SEL2)
	, .JTDO2(JTDO2) 
	);

`ifdef TARGET_MachXO
`elsif TARGET_MachXO2
    wire BSCAN_RESET = ~BSCAN_RESET_N;
`endif

  wire CLK;

`define JTAG_TEST3
`ifdef JTAG_TEST3
  reg r_jtck;
  reg r_jtck_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	r_cs1_prev   <= SEL1 & BSCAN_SHIFT;
	r_cs2_prev   <= SEL2 & BSCAN_SHIFT;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= r_jtck_prev;
	if (r_cs1_prev == (SEL1 & BSCAN_SHIFT))
		r_cs1   <= r_cs1_prev;
	if (r_cs2_prev == (SEL2 & BSCAN_SHIFT))
		r_cs2   <= r_cs2_prev;
    end
  wire JTCK1 = ~r_jtck;
  wire JTCK2 = ~r_jtck;
  wire JTDI  = JTDI_IN;
  wire CS1 = ~r_cs1;
  wire CS2 = ~r_cs2;
`elsif JTAG_TEST2
  reg r_jtck;
  reg r_jtck_prev;
  reg r_jtdi;
  reg r_jtdi_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	r_jtdi_prev <= JTDI_IN;
	r_cs1_prev   <= SEL1 & BSCAN_SHIFT;
	r_cs2_prev   <= SEL2 & BSCAN_SHIFT;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= r_jtck_prev;
	if (r_jtdi_prev == JTDI_IN)
		r_jtdi <= r_jtdi_prev;
	if (r_cs1_prev == (SEL1 & BSCAN_SHIFT))
		r_cs1   <= r_cs1_prev;
	if (r_cs2_prev == (SEL2 & BSCAN_SHIFT))
		r_cs2   <= r_cs2_prev;
    end
  wire JTCK1 = ~r_jtck;
  wire JTCK2 = ~r_jtck;
  wire JTDI  = r_jtdi;
  wire CS1 = ~r_cs1;
  wire CS2 = ~r_cs2;
`else // JTAG_TEST1
  wire JTCK1 = JTCK_IN;
  wire JTCK2 = JTCK_IN;
  wire JTDI  = JTDI_IN;
  wire CS1 = ~(SEL1 & BSCAN_SHIFT);
  wire CS2 = ~(SEL2 & BSCAN_SHIFT);
`endif
  wire [7:0] SPI_DATA;

  spi_echo SPI_ECHO (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK1)
                , .CS(CS1)
                , .MISO(JTDO1)
		, .SPI_DATA(SPI_DATA)
		);
 
  isp ISP (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK2)
                , .CS(CS2)
                , .MISO(JTDO2)
		);

`ifdef HAVE_OSCC
  OSCC osc_internal (.OSC(CLK_INT));
  assign CLK = CLK_INT;
`elsif HAVE_OSCH
  defparam OSCH_inst.NOM_FREQ = "2.08";  //  This is the default frequency 2.08 MHz
  OSCH osc_internal (.OSC(CLK_INT));
  assign CLK = CLK_INT;
`endif

//  reg [24:0] clk_count;
  reg [26:0] clk_count;
  
  always @(posedge CLK)
    begin
         clk_count = clk_count + 1;
    end

`define TEST_02
`ifdef TEST_02
  assign LED[7] = ~clk_count[24];
  assign LED[6] = CS2;
  assign LED[5] = CS1;
  assign LED[4] = ~SPI_DATA[4];
  assign LED[3] = ~SPI_DATA[3];
  assign LED[2] = ~SPI_DATA[2];
  assign LED[1] = ~SPI_DATA[1];
  assign LED[0] = ~SPI_DATA[0];
`else // TEST_01
  // 1st : check led and internal clock
  assign LED[0] = ~(clk_count[26:24] == 0);
  assign LED[1] = ~(clk_count[26:24] == 1);
  assign LED[2] = ~(clk_count[26:24] == 2);
  assign LED[3] = ~(clk_count[26:24] == 3);
  assign LED[4] = ~(clk_count[26:24] == 4);
  assign LED[5] = ~(clk_count[26:24] == 5);
  assign LED[6] = ~(clk_count[26:24] == 6);
  assign LED[7] = ~(clk_count[26:24] == 7);
`endif
endmodule
